# -*- coding: utf-8 -*-
from django.utils.translation import ugettext_lazy as _
from shop_categories.models.defaults.product.base import CategoryProductBase
        
class CategoryProduct(CategoryProductBase):     
    
    class Meta:
        abstract = False
        app_label = 'shop_categories'
        verbose_name = _('Category Product')
        verbose_name_plural = _('Category Products')        
        


from django.conf import settings
from django.core import exceptions

def get_model_string(model_name, setting_format='SHOP_%s_MODEL', fallback_format='shop.%s'):
    """
    Returns the model string notation Django uses for lazily loaded ForeignKeys
    (eg 'auth.User') to prevent circular imports.

    This is needed to allow our crazy custom model usage.
    """
    setting_name = setting_format % model_name.upper().replace('_', '')
    class_path = getattr(settings, setting_name, None)
        
    if not class_path:
        return fallback_format % model_name
    elif isinstance(class_path, basestring):
        parts = class_path.split('.')
        try:
            index = parts.index('models') - 1
        except ValueError, e:
            raise exceptions.ImproperlyConfigured(CLASS_PATH_ERROR % (setting_name, setting_name))
        app_label, model_name = parts[index], parts[-1]
    else:
        try:
            class_path, app_label = class_path
            model_name = class_path.split('.')[-1]
        except:
            raise exceptions.ImproperlyConfigured(CLASS_PATH_ERROR % (setting_name, setting_name))

    return "%s.%s" % (app_label, model_name)
    
def get_category_model_string(model_name):
    return get_model_string(model_name, setting_format='SHOP_CATEGORIES_%s_MODEL', fallback_format='shop_categories.%s')


# -*- coding: utf-8 -*-
"""
This overrides the Category  with the class loaded from the
CATEGORYPRODUCT_CATEGORY_MODEL setting if it exists.
"""
from django.conf import settings
from shop.util.loader import load_class

#==============================================================================
# Extensibility
#==============================================================================
CATEGORY_MODEL = getattr(settings, 'SHOP_CATEGORIES_CATEGORY_MODEL',
    'shop_categories.models.defaults.category.default.Category')
Category = load_class(CATEGORY_MODEL, 'SHOP_CATEGORIES_CATEGORY_MODEL')



from django.conf.urls.defaults import patterns, url
from django.shortcuts import get_object_or_404

from shop.views import ShopListView
from shop.views.product import ProductDetailView
from shop.models.productmodel import Product
from shop_categories.models import Category

class CategoryShopListView(ShopListView):
    
    def get_queryset(self):
        category = get_object_or_404(Category, path=self.kwargs['path'])
        self.category = category
        queryset = super(CategoryShopListView, self).get_queryset()
        return queryset.filter(additional_categories__lft__gte=category.lft, additional_categories__rght__lte=category.rght).distinct()
        
    def get_context_data(self, **kwargs):
        context = super(CategoryShopListView, self).get_context_data(**kwargs)
        context['category'] = self.category
        return context
        
class CategoryProductDetailView(ProductDetailView):
    
    def get_queryset(self):
        queryset = super(CategoryProductDetailView, self).get_queryset()
        return queryset.filter(main_category__path=self.kwargs['path'])
        
    def get_context_data(self, **kwargs):
        context = super(CategoryProductDetailView, self).get_context_data(**kwargs)
        category = get_object_or_404(Category, path=self.kwargs['path'])
        context['category'] = category
        return context
        
urlpatterns = patterns('',
    url(r'^(?P<path>.+)/product/(?P<slug>[0-9A-Za-z-_.//]+)/$',
        CategoryProductDetailView.as_view(),
        name='product_detail'
        ),
    url(r'^(?P<path>.+)/$',
        CategoryShopListView.as_view(model=Product),
        name='product_list'
        ),
    )


# -*- coding: utf-8 -*-
import random
from decimal import Decimal
from shop.models.productmodel import Product # is the overridden CategoryProduct (project.models.product.Product)
from shop_categories.models.categorymodel import Category # is the overridden Category (project.models.category.Category)

from django.test.testcases import TestCase
from django.template.defaultfilters import slugify

def make_category_tree():
    top = Category(name='Top category', slug=slugify('Top category'), active=True)
    top.save()
    level1_first = Category(name='Level1 first', slug=slugify('Level1 first'), active=True, parent=top)
    level1_first.save()
    level1_second = Category(name='Level1 second', slug=slugify('Level1 second'), active=True, parent=top)
    level1_second.save()
    level2_first = Category(name='Level2 first', slug=slugify('Level2 first'), active=True, parent=level1_first)
    level2_first.save()
    level2_first_sub = Category(name='Level2 first sub', slug=slugify('Level2 first sub'), active=True, parent=level2_first)
    level2_first_sub.save()
    level2_second = Category(name='Level2 second', slug=slugify('Level2 second'), active=True, parent=level1_first)
    level2_second.save()
    Category.tree.rebuild()

class CategoryTestCase(TestCase):

    def setUp(self):
        make_category_tree()

    def test_category_unicode(self):
        self.assertEqual(unicode(Category.objects.get(slug='level1-first')), 'Top category> Level1 first')
        
    def test_category_short_title(self):
        self.assertEqual(Category.objects.get(slug='level1-first').short_title(), 'Level1 first')
        
    def test_category_save(self):
        Category.objects.get(slug='level1-first').save()
                
    def test_category_count(self):
        self.assertEqual(Category.objects.count(), 6)
        
    def test_category_leaf_path(self):
        self.assertEqual(Category.objects.get(slug='level2-first-sub').path, 'top-category/level1-first/level2-first/level2-first-sub')
    
    def test_category_leaf_url(self):
        self.assertEqual(Category.objects.get(slug='level2-first-sub').get_absolute_url(), '/shop/catalog/top-category/level1-first/level2-first/level2-first-sub/')
        
class CategoryProductTestCase(TestCase):

    def setUp(self):
        make_category_tree()
        Product(
            name='Product 1',
            slug=slugify('Product 1'),
            active=True,
            unit_price=Decimal(random.randint(50, 1000)),
            main_category=Category.objects.get(slug='level2-first-sub')
        ).save()
        Product(
            name='Product 2',
            slug=slugify('Product 2'),
            active=True,
            unit_price=Decimal(random.randint(50, 1000)),
            main_category=Category.objects.get(slug='level1-first')
        ).save()
        Product(
            name='Product 3',
            slug=slugify('Product 3'),
            active=True,
            unit_price=Decimal(random.randint(50, 1000)),
            main_category=Category.objects.get(slug='level1-second')
        ).save()
                
    def test_product_adds_additional_categories(self):
        p = Product(
            name='Product 4',
            slug=slugify('Product 4'),
            active=True,
            unit_price=Decimal(random.randint(50, 1000)),
            main_category=Category.objects.get(slug='level1-second')
        )
        
        p.save()
        self.assertEqual(p.additional_categories.all()[0].slug, 'level1-second')
        
    def test_product_absolute_url(self):
        self.assertEqual(Product.objects.get(slug='product-1').get_absolute_url(), 
            '/shop/catalog/top-category/level1-first/level2-first/level2-first-sub/product/product-1/')
            
    def test_product_detail(self):
        product_url = Product.objects.get(slug='product-1').get_absolute_url()
        response = self.client.get(product_url)
        self.assertContains(response, '/shop/catalog/top-category/level1-first/level2-first/level2-first-sub/product/product-1/')
                
    def test_list_products_in_category(self):
        category = Category.objects.get(slug='level1-first')
        response = self.client.get(category.get_absolute_url())
        self.assertContains(response, '/shop/catalog/top-category/level1-first/level2-first/level2-first-sub/product/product-1/')
        self.assertContains(response, '/shop/catalog/top-category/level1-first/product/product-2/')
        self.assertNotContains(response, '/shop/catalog/top-category/level1-second/product/product-3/')
        category = Category.objects.get(slug='level1-second')
        response = self.client.get(category.get_absolute_url())
        self.assertNotContains(response, '/shop/catalog/top-category/level1-first/level2-first/level2-first-sub/product/product-1/')
        self.assertNotContains(response, '/shop/catalog/top-category/level1-first/product/product-2/')
        self.assertContains(response, '/shop/catalog/top-category/level1-second/product/product-3/')


from treeadmin.admin import TreeAdmin

class ProductCategoryAdmin(TreeAdmin):
    exclude = ('path', 'name_path')
    list_display = ('name', 'active', 'path')
    prepopulated_fields = {"slug": ("name",)}    



from django.db import models
from shop.models.defaults.bases import BaseProduct
from mptt.fields import TreeForeignKey, TreeManyToManyField
from shop_categories.utils import get_category_model_string

class CategoryProductBase(BaseProduct):
    main_category = TreeForeignKey(get_category_model_string('Category'))
    additional_categories = TreeManyToManyField(get_category_model_string('Category'), related_name='extra_product_categories')

    class Meta:
        abstract = True
        
    @models.permalink    
    def get_absolute_url(self):
        return('product_detail', (), {'slug':self.slug, 'path': self.main_category.path})
        
    def save(self, *args, **kwargs):
        super(CategoryProductBase, self).save(*args, **kwargs)
        self.additional_categories.add(self.main_category)




from django.db import models
from shop_categories.models.defaults.product.base import CategoryProductBase

class Product(CategoryProductBase):
    
    order = models.PositiveIntegerField(null=True, blank=True)

    class Meta:
        abstract = False
        app_label = 'project'


from django.db import models
from django.utils.encoding import force_unicode
from mptt.models import MPTTModel
from mptt.fields import TreeForeignKey

class ProductCategoryBase(MPTTModel):
    
    parent = TreeForeignKey('self', 
        blank=True, 
        null=True, 
        related_name="children", 
        verbose_name='Parent')    
        
    name = models.CharField(max_length=50)
    slug = models.SlugField()
    
    path = models.CharField(max_length=255)
    name_path = models.CharField(max_length=255)

    active = models.BooleanField()
    
    class Meta:
        abstract = True
    
    def __unicode__(self):
        return self.name_path
        
    def short_title(self):
        return self.name
        
    def get_path(self):
        ancestors = []
        if self.parent:
            ancestors = list(self.parent.get_ancestors(include_self=True))
        ancestors = ancestors + [self,]
        return ancestors
        
    def join_path(self, joiner, field, ancestors):
        return joiner.join([force_unicode(getattr(i, field)) for i in ancestors])
        
    def save(self, *args, **kwargs):
        
        ancestors = self.get_path()
        self.path = self.join_path(u'/', 'slug', ancestors)
        self.name_path =  self.join_path(u'> ', 'name', ancestors)
        
        update_descendants = bool(self.pk)
        super(ProductCategoryBase, self).save(*args, **kwargs)
        
        if update_descendants:
            children = list(self.children.all())
            for child in children:
                child.active = self.active
                child.save()
                
    @models.permalink    
    def get_absolute_url(self):
        return('product_list', (), {'path': self.path})            


from setuptools import setup, find_packages
import os

import shop_categories

CLASSIFIERS = [
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Django',
]

setup(
    name='django-shop-categories',
    version=shop_categories.get_version(),
    description='A extendable category app using django-mptt for django-shop',
    long_description=open(os.path.join(os.path.dirname(__file__), 'README.rst')).read(),
    author='Oyvind Saltvik',
    author_email='oyvind.saltvik@gmail.com',
    url='http://github.com/fivethreeo/django-shop-categories/',
    packages=find_packages(),
    package_data={
        'shop_categories': [
            'templates/shop_categories/*',
            'locale/*/LC_MESSAGES/*',
        ]
    },
    classifiers=CLASSIFIERS,
    include_package_data=True,
    zip_safe=False,
    install_requires=['django-shop', 'django-mptt', 'django-treeadmin']
)



# -*- coding: utf-8 -*-
from django.utils.translation import ugettext_lazy as _
from mptt.managers import TreeManager
from shop_categories.models.defaults.category.base import ProductCategoryBase

class Category(ProductCategoryBase):
    tree = TreeManager()
    
    class Meta:
        abstract = False
        app_label = 'shop_categories'
        verbose_name = _('Product Category')
        verbose_name_plural = _('Product Categorys')  


# -*- coding: utf-8 -*-
import os

gettext = lambda s: s

urlpatterns = []

DJANGO_SETTINGS_MODULE = 'shop_categories.test_utils.cli' # this module
ROOT_URLCONF = DJANGO_SETTINGS_MODULE

def configure(**extra):
    from django.conf import settings
    os.environ['DJANGO_SETTINGS_MODULE'] = DJANGO_SETTINGS_MODULE
    defaults = dict(
        CACHE_BACKEND = 'locmem:///',
        DEBUG = True,
        TEMPLATE_DEBUG = True,
        DATABASE_SUPPORTS_TRANSACTIONS = True,
        DATABASES = {
            'default': {
                'ENGINE': 'django.db.backends.sqlite3',
                'NAME': ':memory:',
            }
        },
        SITE_ID = 1,
        USE_I18N = True,
        MEDIA_ROOT = '/media/',
        STATIC_ROOT = '/static/',
        MEDIA_URL = '/media/',
        STATIC_URL = '/static/',
        ADMIN_MEDIA_PREFIX = '/static/admin/',
        EMAIL_BACKEND = 'django.core.mail.backends.locmem.EmailBackend',
        SECRET_KEY = 'key',
        TEMPLATE_LOADERS = (
            'django.template.loaders.filesystem.Loader',
            'django.template.loaders.app_directories.Loader',
            'django.template.loaders.eggs.Loader',
        ),
       TEMPLATE_CONTEXT_PROCESSORS = [
            "django.contrib.auth.context_processors.auth",
            "django.core.context_processors.i18n",
            "django.core.context_processors.debug",
            "django.core.context_processors.request",
            "django.core.context_processors.media",
            'django.core.context_processors.csrf',
            "django.core.context_processors.static"
        ],
        MIDDLEWARE_CLASSES = [
            'django.contrib.sessions.middleware.SessionMiddleware',
            'django.contrib.auth.middleware.AuthenticationMiddleware',
            'django.contrib.messages.middleware.MessageMiddleware',
            'django.middleware.common.CommonMiddleware',
            'django.middleware.doc.XViewMiddleware',
            'django.middleware.csrf.CsrfViewMiddleware'
        ],
        INSTALLED_APPS = [
            'django.contrib.auth',
            'django.contrib.contenttypes',
            'django.contrib.sessions',
            'django.contrib.admin',
            'django.contrib.sites',
            'django.contrib.staticfiles',
            'treeadmin',
            'shop_categories',
            'shop_categories.test_utils.project'
        ],
        LANGUAGE_CODE = "en",
        LANGUAGES = (
            ('en', gettext('English')),
            ('fr', gettext('French')),
            ('de', gettext('German')),
            ('pt-BR', gettext("Brazil")),
            ('nl', gettext("Dutch")),
        ),
        SOUTH_TESTS_MIGRATE = False,
        ROOT_URLCONF = ROOT_URLCONF,
        SHOP_PRODUCT_MODEL = 'shop_categories.test_utils.project.models.product.Product',
        SHOP_CATEGORIES_CATEGORY_MODEL = 'shop_categories.test_utils.project.models.category.Category'
    )
    defaults.update(extra)
    settings.configure(**defaults)